@@ -18,8 +18,8 @@ from trac.util.datefmt import datetime_now, format_datetime, utc
 from trac.util.html import tag
 from trac.wiki.macros import WikiMacroBase
 
-revision = "$Rev$"
-url = "$URL$"
+revision = "$Rev: 17657 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/trunk/sample-plugins/Timestamp.py $"
 
 
 class TimestampMacro(WikiMacroBase):
